Unit SapDLG;   // taken from  system Unit "FileCtrl" and rearranged

interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, IniFiles, Buttons, ExtCtrls, StdCtrls, ComCtrls, FileCtrl,
  SapTools;

Procedure Anons(m:string; mt:TMsgDlgType);
Function  Confirm(m:string; mt:TMsgDlgType):boolean;
Function  ChooseDirectory(Title:string; InitDir: string) : string;
Function  ChooseFileR(Title:string; InitFile: string; Filter:string) : string;
Function  ChooseFileW(Title:string; InitFile: string; Filter:string) : string;

implementation

Type TChooseType = (ctDir,ctFileR,ctFileW);

  TPathLabel = class(TCustomLabel)
  protected
    procedure Paint; override;
  public
    constructor Create(AnOwner: TComponent); override;
  published
    property Alignment;
    property Transparent;
  end;

  TSelectDirDlg = class(TForm)
    DirList: TDirectoryListBox;
    DirEdit: TEdit;
    DriveList: TDriveComboBox;
    DirLabel: TPathLabel;
    FreeLabel: TLabel;
    FilterComboBox:TFilterComboBox;
    OKButton: TBitBtn;
    CancelButton: TBitBtn;
    NetButton: TBitBtn;
    FileList: TFileListBox;
    procedure DirListClick(Sender: TObject);
    procedure DirListChange(Sender: TObject);
    procedure FileListChange(Sender: TObject);
    procedure FileListDblClick(Sender: TObject);
    procedure DriveListChange(Sender: TObject);
    procedure NetClick(Sender: TObject);
  private
    { Private declarations }
    Ready:boolean;
    FChooseType: TChooseType;
    procedure SetDirectory(const Value: string);
    function GetDirectory: string;
  public
    { Public declarations }
    Constructor Create(Title:string; ChooseType:TChooseType; InitPath: string; Filt:string);
    property Directory: string read GetDirectory write SetDirectory;
  end;

Function TrailingSlash(s:string):string;
begin Result:=s;  if s='' then Exit;
  if s[Length(s)]<>'\' then Result:=s+'\';
end;

{ TPathLabel }

constructor TPathLabel.Create(AnOwner: TComponent);
begin   inherited Create(AnOwner);
  WordWrap := False;  AutoSize := False;  ShowAccelChar := False;
end;

procedure TPathLabel.Paint;
const Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var   Rect: TRect;  Temp: String;
begin  with Canvas do
  begin Rect := ClientRect;
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(Rect);
    end;
    Brush.Style := bsClear;
    Temp := MinimizeName(Caption, Canvas, Rect.Right - Rect.Left);
    DrawText(Canvas.Handle, PChar(Temp), Length(Temp), Rect, DT_NOPREFIX or Alignments[Alignment]);
  end;
end;

{ TSelectDirDlg }

Const L1=6; T1=40; H1=24; W1=70; GapH=10;
      WD=200; WF=150;
Constructor TSelectDirDLG.Create(Title:string; ChooseType:TChooseType; InitPath: string; Filt:string);
begin Ready:=false;
  inherited CreateNew(Application);
  Caption := Title;  FChooseType:=ChooseType;
  ClientHeight := 300+H1+GapH;  ClientWidth := 3*L1+WD+WF;
  BorderStyle := bsDialog;  Position := poScreenCenter;
// -----------------------------------------
  DirEdit := TEdit.Create(Self);
  with DirEdit do
  begin Parent:=Self; TabOrder:=0;
    SetBounds(L1,L1, WD+WF+L1,H1);
    Text:=InitPath; Visible:=(FChooseType=ctFileW);
  end;
  DirLabel := TPathLabel.Create(Self);
  with DirLabel do
  begin Parent := Self; BoundsRect := DirEdit.BoundsRect;
    Visible:=(FChooseType<>ctFileW);
  end;
  FilterComboBox := TFilterComboBox.Create(Self);
  with FilterComboBox do
  begin Parent:=Self; TabOrder:=0;
    SetBounds(2*L1+WD,T1, WF,H1);
    Visible:=(Pos('|',Filt)>0);  OnChange:=DirListChange;
  end;
  FreeLabel := TLabel.Create(Self);
  with FreeLabel do
  begin Parent := Self;  Visible:=not FilterComboBox.Visible;
    BoundsRect := FilterComboBox.BoundsRect;
  end;
  NetClick(nil);                            // to create initial DriveList
  DirList := TDirectoryListBox.Create(Self);
  with DirList do
  begin Parent:=Self; TabOrder:=1; TabStop:=True; ItemHeight:=17;
    IntegralHeight:=True; OnChange:=DirListChange;  OnClick:=DirListClick;
    SetBounds(L1, T1+H1+GapH, WD,Self.ClientHeight-T1-2*H1-3*GapH);
  end;
  FileList := TFileListBox.Create(Self);
  with FileList do
  begin Parent:=Self; TabOrder:=6; TabStop:=True; ItemHeight:=17;
    IntegralHeight:=True; OnChange:=FileListChange;
    if FChooseType<>ctDir then OnDblClick:=FileListDblClick;
    SetBounds(2*L1+WD,T1+H1+GapH, WF,Self.ClientHeight-T1-2*H1-3*GapH);
    FileType:=[ftNormal];
    if FChooseType=ctDir then begin ItemHeight:=13; Font.Color:=clGrayText; end;
  end;
  NetButton := TBitBtn.Create(Self);
  with NetButton do
  begin Parent:=Self; TabOrder:=3;  OnClick:=NetClick;
    if Language=Russian then Caption:='...' else Caption:='Network...';
    Hint:='     Windows-'; ShowHint:=(Language=Russian);
    SetBounds(L1,T1+H1+2*GapH+DirList.Height, W1,H1);
  end;
  OKButton := TBitBtn.Create(Self);
  with OKButton do
  begin Parent := Self; TabOrder:=4; Kind:=bkOK;
    Hint:=',   '; ShowHint:=(Language=Russian);
    SetBounds(DirList.Left+WD-W1,NetButton.Top, W1,H1);
  end;
  CancelButton := TBitBtn.Create(Self);
  with CancelButton do
  begin Parent := Self;  ModalResult:=mrCancel;  Kind:=bkCancel;
    SetBounds(FileList.Left+WF-W1,NetButton.Top, W1,H1);
    TabOrder:=5; Cancel:=True; if Language=Russian then Caption:='';
  end;
  DirListChange(Self);
  Directory := InitPath;
  if FChooseType=ctDir then ActiveControl := DirList else ActiveControl:=FileList;
  if Filt='' then FilterComboBox.Filter:='*.*' else FilterComboBox.Filter:=Filt;
  Ready:=true;
end;

Procedure TSelectDirDlg.DriveListChange(Sender: TObject);
var i:int64;
begin
  if Assigned(DirList) then DirList.Drive := DriveList.Drive;
  i:=DiskFree((Ord(DriveList.Drive)-Ord('A')+1) mod 32); if i>0 then i:=i shr 10 else i:=0;
  FreeLabel.Caption:=IntToStr(i);
  Case Language of
    English: FreeLabel.Caption:='   Free: '+FreeLabel.Caption+' kBytes.';
    Russian: FreeLabel.Caption:='   : '+FreeLabel.Caption+' kB';
  end;
  DirListChange(Sender);
end;

procedure TSelectDirDlg.DirListClick(Sender: TObject);
begin  if Assigned(DirList) then DirList.OpenCurrent;  end;

procedure TSelectDirDlg.DirListChange(Sender: TObject);
begin
  if not Assigned(DirList) then Exit;
  if not Assigned(FileList) then Exit;
  if not Assigned(DirLabel) then Exit;
  if not Assigned(DirEdit) then Exit;
  DirLabel.Caption := TrailingSlash(DirList.Directory);
  FileList.Directory := DirList.Directory;
  FileList.ItemIndex:=-1;
  FileList.Mask:=FilterComboBox.Mask;
  With DirEdit do
  begin Text := DirLabel.Caption; SelectAll;
    if Ready and Visible then SetFocus;
  end;
end;

procedure TSelectDirDlg.FileListChange(Sender: TObject);
begin  if FChooseType=ctDir then begin FileList.ItemIndex:=-1; Exit; end;
  if FileList.FileName<>'' then DirLabel.Caption := FileList.FileName;
  FileList.Directory := DirList.Directory;
  With DirEdit do
  begin Text := DirLabel.Caption; SelectAll;
    if Ready and Visible then SetFocus;
  end;
  if (Directory<>'') and (FileList.ItemIndex>=0) then Case Language of
    English: FreeLabel.Caption:='   Size: '+IntToStr(SizeOfFile(Directory))+' bytes.';
    Russian: FreeLabel.Caption:='   : '+IntToStr(SizeOfFile(Directory))+' .';
  end;
end;

procedure TSelectDirDlg.FileListDblClick(Sender: TObject);
begin  FileListChange(Sender);  ModalResult:=mrOK; end;

procedure TSelectDirDlg.SetDirectory(const Value: string);
var
  Temp: string;
begin if Value='' then Exit;
  Temp:=ExpandFileName(Value);
  DriveList.Drive := Temp[1];
  if DirectoryExists(Temp) then Temp:=Temp+'\' else Temp := ExtractFilePath(Temp);
  try
    DirList.Directory := Copy(Temp, 1, Length(Temp) - 1);
  except
    on EInOutError do begin GetDir(0,Temp); DriveList.Drive:=Temp[1]; DirList.Directory:=Temp; end;
  end;
end;

function TSelectDirDlg.GetDirectory: string;
begin  Result := DirLabel.Caption;  end;

procedure TSelectDirDlg.NetClick(Sender: TObject);
begin  if Assigned(Sender) then  WNetConnectionDialog(Handle, WNTYPE_DRIVE);
  DriveList.Free;   DriveList := TDriveComboBox.Create(Self);
  with DriveList do
  begin Parent:=Self;  TabOrder:=2;  OnChange:=DriveListChange;  TextCase:=tcUpperCase;
    SetBounds(L1,T1, WD,H1);
  end;
end;

function ChooseFileDir(Title:string; ChooseType:TChooseType; InitPath: string; Filter:string) : string;
var D: TSelectDirDlg; h,p:string;
begin  h:=GetCurrentDir; p:=InitPath;
  if p='' then if DirectoryExists('A:\') then p:='A:' else if DirectoryExists('B:\') then p:='B:' else p:='C:';
  D := TSelectDirDlg.Create(Title,ChooseType,p,Filter);
  try
    if Screen.PixelsPerInch <> 96 then
    begin   // scale to screen res
      D.ScaleBy(Screen.PixelsPerInch, 96);
      D.FileList.ParentFont := True;
      D.Left := (Screen.Width div 2) - (D.Width div 2);
      D.Top := (Screen.Height div 2) - (D.Height div 2);
      D.FileList.Font.Color := clGrayText;
    end;
    if D.ShowModal = mrOK  then Result:={D.Directory} D.DirEdit.Text else Result:='';
    if (ChooseType<>ctDir) and (TrailingSlash(Result)=Result) then Result:='';
  finally
    D.Free;  ChDir(h);     // not change current Dir !!!
  end;
end;

function  ChooseDirectory(Title:string; InitDir: string) : string;
begin Result:=ChooseFileDir(Title,ctDir,InitDir,''); end;

Function  ChooseFileR(Title:string; InitFile: string; Filter:string) : string;
begin Result:=ChooseFileDir(Title,ctFileR,InitFile,Filter); end;

Function  ChooseFileW(Title:string; InitFile: string; Filter:string) : string;
begin Result:=ChooseFileDir(Title,ctFileW,InitFile,Filter); end;

{ -----------------------------------------------------------------------------
  Anons(m:string; mt:TMsgDlgType) -  Message M   .
         ,         .
    MT -   MessageDlg,   ,   Language.
  Confirm(---"---) -   ,   2 : Yes or No (  )
       true   "Yes"
}
Type TOwnMessageForm = class(TForm)
  procedure Key(sender:Tobject; var key:word; s:TShiftState);
  procedure Click(sender:Tobject);
end;
procedure TOwnMessageForm.Key(sender:Tobject; var key:word; s:TShiftState);
begin ModalResult:=mrOK; end;
procedure TOwnMessageForm.Click(sender:Tobject);
begin ModalResult:=mrOK; end;

Procedure Anons(m:string; mt:TMsgDlgType);
var i:integer;
begin
  With TOwnMessageForm(CreateMessageDialog(m,mt,[])) do
  begin Color:=clAqua;
    OnKeyDown:=Key;  OnClick:=Click;
    for i:=0 to ComponentCount-1 do TLabel(Components[i]).OnClick:=Click;
    Case Language of
      English:;
      Russian: Case mt of
                 mtWarning:      Caption:=' :';
                 mtError:        Caption:=' :';
                 mtInformation:  Caption:='   :';
                 mtConfirmation: Caption:=',  :';
                 mtCustom:       Caption:=Application.Title;
               end;
    end;
    Height:=Height-40;
    ShowModal; Free;
  end;
end;

Function Confirm(m:string; mt:TMsgDlgType):boolean;
var i:integer;
begin
  With TOwnMessageForm(CreateMessageDialog(m,mt,[mbYes,mbNo])) do
  begin Color:=clAqua;
    Case Language of
      English:;
      Russian: Case mt of
                 mtWarning:      Caption:=' :';
                 mtError:        Caption:=' :';
                 mtInformation:  Caption:='   :';
                 mtConfirmation: Caption:='   :';
                 mtCustom:       Caption:=Application.Title;
               end;
    end;
    for i:=0 to ComponentCount-1 do
      if Components[i] is TButton then Case Language of
        English:;
        Russian: if TButton(Components[i]).Name='Yes'
                    then TButton(Components[i]).Caption:=''
                    else TButton(Components[i]).Caption:='';
      end;
    Result:=(ShowModal=mrYes); Free;
  end;
end;


end.
