unit SAPTools;

interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, IniFiles, Registry, Buttons, ExtCtrls, StdCtrls, ComCtrls, ShellAPI;

Type  TLanguage = (Russian,English); { maybe other - why not }
      TInt  = Longint;
      TReal = Double;
Type Fcomp2item = Function( P:pointer; n1,n2:integer ):boolean;

  TOwnFont = class
  private
    FName:string;
  public
    Constructor Create(aName:string);
    Destructor Destroy; override;
    Property Name:string read FName;
  end;

Const Language:TLanguage = English;
      NumberRepresentationSize:integer = 10;
      DefaultMantisSize:integer = 4;
      SmallEnoughToBeReplacedByZero:TReal=0;

Function RealToStr(r:TReal; m:integer):string;
Function RealToStrFix(r:TReal; n:integer; m:integer):string;
Function Replicate(s:string; k:integer):string;
Function GetFragment(var s:string):string;
Function GetRealNumber(var s:string):TReal;
Function GetIntegerNumber(var s:string):integer;
Function StrToReal(s:string):TReal;
Function ExpandString(s:string; n:integer):string;
Function LeftFromImage(Source:string; Image:string):string;
Procedure Sh_Sort( P:pointer; N:integer; Comp:Fcomp2item );
Function  LatinTranscription(s:string):string;
Function  RusTranscription(s:string):string;
Function  ShortFileName(s:string):string;
Function  SizeOfFile(filename:string):integer;
Function ExecuteFile(FileName, Params, DefaultDir: string): integer;
Procedure ExecuteDosCommand(CommandLine,DefaultDir:string);
Function GetHomeDir:string;                     // Where is my EXE-file
Procedure SetHScroll(ListBox:TListBox);
Procedure Dos2Win(var s:string);
Procedure Win2Dos(var s:string);
Procedure Delay(msecs : Longint);
Function ClientValidation:string;               // see install.pas
Procedure StampOut;

implementation

uses ShlObj, ComObj, ActiveX;

Function Replicate(s:string; k:integer):string;
begin Result:=''; While k>0 do begin Dec(k); Result:=Result+s; end;  end;

Function LeftFromImage(Source:string; Image:string):string;
var i:integer;
begin
  i:=Pos(Image,Source);
  if i=0 then Result:=Source else Result:=Copy(Source,1,i-1);
end;

Function RealToStr(r:TReal; m:integer):string;
//          
var i:integer;
begin DecimalSeparator:='.'; // DecimalSeparator can be changed from outside!
  Result:=FloatToStrF(r,ffGeneral,NumberRepresentationSize,2);
  if m<0 then Exit;
  i:=Pos(',',Result); if i>0 then Result[i]:='.';
  i:=Pos('.',Result); if i=0 then Exit;
  if Pos('E',Result)>0 then Exit;
  if m=0 then Dec(i);
  Result:=Copy(Result,1,i+m);
end;

Function RealToStrFix(r:TReal; n:integer; m:integer):string;
//          
//        N 
var i:integer; rr:TReal;
begin
  if abs(r)<SmallEnoughToBeReplacedByZero then rr:=0 else rr:=r;
  Result:=RealToStr(rr,m);
  i:=Length(Result); if n<2 then Exit;
  if i>n then Result:='*'+Copy(Result,1,n-1) else Result:=Replicate(' ',n-i)+Result;
end;

Function GetFragment(var s:string):string;
var i:integer;
begin
  s:=Trim(s);  i:=Pos(' ',s);
  if i=0 then begin Result:=s; s:=''; end else
    begin Result:=Copy(s,1,i-1); Delete(s,1,i); end;
end;

Function GetRealNumber(var s:string):TReal;
var i:integer;
    t:string;
begin DecimalSeparator:='.'; // DecimalSeparator can be changed from outside!
  t:=GetFragment(s);
  Result:=0; if t='' then Exit;
  i:=Pos(',',t); if i>0 then t[i]:='.';
  if Pos('.',t)=Length(t) then t:=t+'0';  { Val gives error for "0."  ! }
  Val(t,Result,i); if i=0 then Exit;
  Case Language of
    Russian:ShowMessage('... '+t+'  '+s+Chr(13)+'   ^     !');
    English:ShowMessage('... '+t+'  '+s+Chr(13)+'   ^  Illegal number !');
  end;
end;

Function GetIntegerNumber(var s:string):integer;
begin Result:=Round(GetRealNumber(s)); end;

Function StrToReal(s:string):TReal;
var s1:string;
begin s1:=s; Result:=GetRealNumber(s1); end;

Function ExpandString(s:string; n:integer):string;
begin Result:=s;
  While Length(Result)<n do Result:=Result+' ';
  if Length(Result)>n then Result:=Copy(Result,1,n);
end;

Procedure SetHScroll(ListBox:TListBox);
//       ListBox
Var i,w,m:integer;
begin m:=0;
  for I := 0 to ListBox.Items.Count - 1 do
  begin
    w:=ListBox.Canvas.TextWidth(ListBox.Items.Strings[I]);
    if m<w then m:=w;
  end;
  SendMessage(ListBox.Handle,LB_SETHORIZONTALEXTENT,((m+2)*Screen.Width) div 640, 0);
end;

procedure Delay(msecs : Longint);
var FirstTick : Longint;
begin FirstTick := LongInt(GetTickCount);
repeat
  Application.ProcessMessages;
until LongInt(GetTickCount) - FirstTick > msecs;
end;

Procedure Sh_Sort( P:pointer; N:integer; Comp:Fcomp2item );
Var step,i,j,k:integer;
begin if N<2 then Exit;
  step:=4; if N>25 then Repeat step:=3*step+1; Until step>=N;
  Repeat step:=(step-1) div 3;  for i:=1 to step do  { simple insert inside }
    begin j:=i;
      Repeat Inc(j,step); k:=j; if k<=N then
      	Repeat Dec(k,step); Until Comp(P,k,k+step) or (k=i);
      Until j>N-step;
    end;
  Until step=1;
end;

Function  LatinTranscription(s:string):string;
var i:integer;
begin Result:='';
  for i:=1 to Length(s) do  Case s[i] of
      ' '  : Result:=Result+'_';     //       
    '','': Result:=Result+'A';
    '','': Result:=Result+'B';
    '','': Result:=Result+'V';
    '','': Result:=Result+'G';
    '','': Result:=Result+'D';
    '','': Result:=Result+'E';
    '','': Result:=Result+'H';
    '','': Result:=Result+'Z';
    '','': Result:=Result+'I';
    '','': Result:=Result+'J';
    '','': Result:=Result+'K';
    '','': Result:=Result+'L';
    '','': Result:=Result+'M';
    '','': Result:=Result+'N';
    '','': Result:=Result+'O';
    '','': Result:=Result+'P';
    '','': Result:=Result+'R';
    '','': Result:=Result+'S';
    '','': Result:=Result+'T';
    '','': Result:=Result+'U';
    '','': Result:=Result+'F';
    '','': Result:=Result+'X';
    '','': Result:=Result+'C';
    '','': Result:=Result+'c';
    '','': Result:=Result+'s';
    '','': Result:=Result+'c';
    '','': Result:=Result+'y';
    '','': Result:=Result+'Y';
    '','': Result:=Result+'y';
    '','': Result:=Result+'e';
    '','': Result:=Result+'W';
    '','': Result:=Result+'Q';
      else   Result:=Result+UpCase(s[i]);
  end;
end;

Function  RusTranscription(s:string):string;
var i:integer;
begin Result:='';
  for i:=1 to Length(s) do  Case s[i] of
    'a','A': Result:=Result+'';
    'b','B': Result:=Result+'';
    'c','C': Result:=Result+'';
    'e','E': Result:=Result+'';
    'h','H': Result:=Result+'';
    'k','K': Result:=Result+'';
    'm','M': Result:=Result+'';
    'o','O': Result:=Result+'';
    'p','P': Result:=Result+'';
    't','T': Result:=Result+'';
    'x','X': Result:=Result+'';
      else   Result:=Result+s[i];
  end;
  Result:=AnsiUpperCase(Result);
end;

Function ShortFileName(s:string):string;
var i:integer;
begin
  Result:=AnsiUpperCase(Copy(LatinTranscription(Trim(s)),1,8));
  for i:=1 to Length(Result) do
    if Pos(Result[i],' .*?|<>":')>0 then Result[i]:='_';
end;

Function  SizeOfFile(filename:string):integer;
var f:file of byte;
begin Result:=-1;
  {$I-} AssignFile(f,filename); Reset(f); {$I+}
  if IoResult<>0 then Exit;
  Result:=FileSize(f);
  CloseFile(f);
end;

Procedure Dos2Win(var s:string);
var i,k:integer;
begin
  for i:=1 to Length(s) do
  begin k:=Ord(s[i]);
    if (k>=$E0) and (k<=$EF) then s[i]:=Chr(k+16)
      else if(k>=$80) and (k<=$AF) then s[i]:=Chr(k+64)
        else Case k of
               $F0: s[i]:=Chr($A8);
               $F1: s[i]:=Chr($B8);
             end;
   end;
end;

Procedure Win2Dos(var s:string);
var i:integer;
begin
  for i:=1 to Length(s) do
  if Ord(s[i]) > $BF then
    if Ord(s[i]) > $EF then s[i]:=Chr(Ord(s[i])-16)
        else s[i]:=Chr(Ord(s[i])-64)
     else Case Ord(s[i]) of
            $A8: s[i]:=Chr($F0);
            $B8: s[i]:=Chr($F1);
          end;
end;

Function GetHomeDir:string;  { Where is my EXE-file }
begin Result:=ExtractFileDir(Application.Exename); end;

   { TOwnFont }

Constructor TOwnFont.Create(aName:string);
begin
  Inherited Create;
  FName:=GetHomeDir+'\'+aName+'.fon';
  AddFontResource( PChar(FName) );   // set own font
  SendMessage(HWND_BroadCast, WM_FontChange,0,0);
end;

Destructor TOwnFont.Destroy;
begin
  RemoveFontResource(PChar(Fname));   // kill own font
  SendMessage(HWND_BroadCast, WM_FontChange,0,0);
  Inherited Destroy;
end;


function ExecuteFile(FileName, Params, DefaultDir: string): integer;
var zFileName, zParams, zDir: array[0..255] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), SW_SHOWNORMAL);
  if Result<33 then ShowMessage(' '+IntToStr(Result)+'   '#13+
         '  '+FileName+'  '+Params);
end;

Procedure ExecuteDosCommand(CommandLine,DefaultDir:string);
Const RedirectedOutput='output.dos';
var rout:string; t,n:integer;
begin t:=0;
  rout:=GetHomeDir+'\'+RedirectedOutput;  DeleteFile(rout);
  n:=ExecuteFile(GetHomeDir+'\install.exe','EXECUTE '+CommandLine+' > '+rout,DefaultDir);
  Repeat t:=t+1000; Delay(1000); Until DeleteFile(rout) or (t>60000);
  SetCurrentDir(GetHomeDir);
//  if n>32 then TerminateThread(n,0);    // -   NT ???
end;

Const ownmark=' Copyright (C) SapSoft. 2001. V3.16 ';   // Attention: Length fixed !!!
Const Crypt='\qsweecrutryiutiyo.pdaastdifngshtjaklllz.xecxvebCNHMEQCWKE en otk6i7l2l0 ';
Type
   TInstaller=record
      Signature:string[Length(ownmark)];
      ClientName:string[80];
      ProductName:string[80];
      WrittenBy:string[80];
      KeyWord:string[20];
      Computer:array[0..32] of byte;
      InstallLimit,RunLimit,TrialPeriod:SmallInt;
      PrevDat,LastDat:LongInt;
    end;
Var Installer:TInstaller;
    buf:array[0..SizeOf(TInstaller)-1] of byte absolute Installer;

Function ClientValidation:string;     // see install.pas
Const MinVol=1000000;
Var f:file;
    s:string; i,n:integer;
begin Result:='';
  s:=''; i:=1; While i<Length(Crypt) do begin s:=s+Crypt[i]; Inc(i,2); end;
  if (DiskFree(0)<MinVol) or (DiskFree(3)<MinVol) then
    ShowMessage('       .'#13+
                '     , '#13+
                '   !     1 MB !');
  if not FileExists(GetHomeDir+'\'+Copy(s,14,11)) then Exit;
// comment 3 lines below to put security off
  n:=ExecuteFile(Copy(s,14,11),Copy(s,25,12),GetHomeDir);
  Delay(2000);                        // enough to Validate !?
  if FileExists(GetHomeDir+'\'+Copy(s,1,13)) then Exit;

  if n>32 then TerminateThread(n,0);
 {$I-} Assign(f,Copy(s,14,11)); Reset(f,1); {$I+}
  if IoResult<>0 then Exit;
  Seek(f,Filesize(f)-512);  BlockRead(f,Buf,SizeOf(Buf)); CloseFile(f);
  for i:=0 to SizeOf(TInstaller)-1 do Buf[i]:=(Buf[i] xor ((3*i+7) mod 256));
  s:=Installer.ClientName;   Dos2Win(s);  Installer.ClientName:=s;
  s:=Installer.ProductName;  Dos2Win(s);  Installer.ProductName:=s;
  s:=Installer.KeyWord;      Dos2Win(s);  Installer.KeyWord:=s;
  s:=Installer.WrittenBy;    Dos2Win(s);  Installer.WrittenBy:=s;
  Installer.Signature:=ownMark;  Result:=Installer.ClientName;
end;

Procedure StampOut;
var s:string;
begin
  ClientValidation;
  With Installer do
  begin s:='Run Limit:'+IntToStr(RunLimit)+'  Trial Period:'+IntToStr(TrialPeriod);
    if LastDat=0 then s:='' else s:=s+'   ( only '+IntToStr(LastDat-PrevDat)+' days left! )'#13;
    ShowMessage(Trim(ProductName)+'    written by  '+Trim(WrittenBy)+' '#13+
                 s+'Client Name: '+Trim(ClientName));
  end;
end;

end.



